//*********************************************************
//GLFONT.CPP -- glFont routines
//Copyright (c) 1998 Brad Fish
//See glFont.txt for terms of use
//November 10, 1998
//*********************************************************

#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include "glfont.h"

//*********************************************************
//Variables
//*********************************************************

//Current font
GLFONT *glFont;

//*********************************************************
//Functions
//*********************************************************
GLFONT* glFontCreate (char *FileName, int Tex)
{
	FILE *Input;
	char *TexBytes;
	int Num;
	GLFONT *Font;
	Font=(GLFONT *)malloc(sizeof(GLFONT));
	//Open font file
	if ((Input = fopen(FileName, "rb")) == NULL)
		return FALSE;
	//printf("start\n");
	//Read glFont structure
	fread(Font, sizeof(GLFONT), 1, Input);
	//printf("input\n");
	//Save texture number
	Font->Tex = Tex;

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;
	
	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR *)malloc(
		sizeof(GLFONTCHAR) * Num)) == NULL)
		return FALSE;
	//printf("charalloc\n");
	//Read glFont characters
	fread(Font->Char, sizeof(GLFONTCHAR), Num, Input);
	//printf("charread\n");
	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * 2;
	
	//Allocate memory for texture data
	if ((TexBytes = (char *)malloc(Num)) == NULL)
		return FALSE;
	//printf("texalloc\n");
	//Read texture data
	fread(TexBytes, sizeof(char), Num, Input);
	//printf("texread\n");
	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font->Tex); 
	//printf("texbind\n");
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		GL_CLAMP);
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR); 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_NEAREST); 
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,GL_MODULATE);  
	//printf("texsettings\n");
	//Create texture
	glTexImage2D(GL_TEXTURE_2D, 0, 2, Font->TexWidth,Font->TexHeight, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, (void *)TexBytes);
	//gluBuild2DMipmaps(GL_TEXTURE_2D, 2, Font->TexWidth,Font->TexHeight, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, (void *)TexBytes);
	//printf("teximage\n");
	//Clean up
	free(TexBytes);
	fclose(Input);

	//Return pointer to new font
	return Font;
}
//*********************************************************
void glFontDestroy (GLFONT *Font)
{
	//Free character memory
	free(Font->Char);
}
//*********************************************************
void glFontBegin (GLFONT *Font)
{
	//Save pointer to font structure
	if (Font->Char != NULL)
		glFont = Font;
	else
		glFont = NULL;
	glEnable(GL_TEXTURE_2D);
	//Bind to font texture
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
}
//*********************************************************
void glFontEnd (void)
{
	//Font no longer current
	glFont = NULL;
	glDisable(GL_TEXTURE_2D);
}
//*********************************************************
void glFontTextOut (char *String)
{
	int Length, i;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;
	
	//Get length of string
	Length = strlen(String);
	
	//Begin rendering quads
	glBegin(GL_QUADS);
	float x;
	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];
		
		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex2f(x,0);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex2f(x,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex2f(x + Char->dx,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex2f(x + Char->dx,0);
	
		//Move to next character
		x += Char->dx;
	}
	//Stop rendering quads
	glEnd();
}

void glFontTextOutc (char *String)
{
	int Length, i;
	float l;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;
	
	//Get length of string
	Length = strlen(String);
	
	//Begin rendering quads
	glBegin(GL_QUADS);
	for (i = 0; i < Length; i++)
	{	
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];
		l+=Char->dx;
	}
	float x=-(l/2);
	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];
		
		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex2f(x,0);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex2f(x,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex2f(x + Char->dx,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex2f(x + Char->dx,0);
	
		//Move to next character
		x += Char->dx;
	}
	//Stop rendering quads
	glEnd();
}
void glFontTextOutr (char *String)
{
	int Length, i;
	float l;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont 
	if (glFont == NULL)
		return;
	
	//Get length of string
	Length = strlen(String);
	
	//Begin rendering quads
	glBegin(GL_QUADS);
	for (i = 0; i < Length; i++)
	{	
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];
		l+=Char->dx;
	}
	float x=-(l);
	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];
		
		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex2f(x,0);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex2f(x,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex2f(x + Char->dx,-Char->dy);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex2f(x + Char->dx,0);
	
		//Move to next character
		x += Char->dx;
	}
	//Stop rendering quads
	glEnd();
}
//*********************************************************

//End of file



